from nonebot import on_command, on_regex
from nonebot.permission import SUPERUSER
from nonebot.adapters.cqhttp import Bot, GROUP, GROUP_ADMIN, GROUP_OWNER, GroupMessageEvent
from nonebot.typing import T_State
from nonebot.log import logger
from .utils import eating_manager, Meals, GROUPS_ID
from nonebot import require, get_bot

greating_helper = require("nonebot_plugin_apscheduler").scheduler
eating_helper = require("nonebot_plugin_apscheduler").scheduler

__what2eat_version__ = "0.1.3"
plugin_notes = f'''
今天吃什么 v{__what2eat_version__}
[xx吃xx]    问bot恰什么
[添加 xx]   添加菜品至群菜单
[移除 xx]   从菜单移除菜品
[加菜 xx]   添加菜品至基础菜单
[菜单]      查看群菜单
[基础菜单]      查看基础菜单
[开启/关闭小助手] 开启/关闭按时吃饭小助手'''.strip()

plugin_help = on_command("吃什么帮助", permission=GROUP, priority=15, block=True)
what2eat = on_regex(r"^(今天|[早中午晚][上饭餐午]|早上|夜宵|今晚)吃(什么|啥|点啥)", permission=GROUP, priority=15, block=True)
add_group = on_command("添加", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
remove_food = on_command("移除", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
add_basic = on_command("加菜", permission=SUPERUSER, priority=15, block=True)
show_group = on_command("菜单", aliases={"群菜单", "查看菜单"}, permission=GROUP, priority=15, block=True)
show_basic = on_command("基础菜单", permission=SUPERUSER, priority=15, block=True)
switch_greating = on_regex(r"(开启|关闭)小助手", permission=SUPERUSER, priority=15, block=True)

@plugin_help.handle()
async def _(bot: Bot):
    await plugin_help.finish(plugin_notes)

@what2eat.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    msg = eating_manager.get2eat(event)
    await what2eat.finish(msg)

@add_group.handle()
async def _(bot: Bot, event: GroupMessageEvent, state: T_State):
    args = event.get_plaintext().strip().split()
    if not args:
        await add_group.finish("还没输入你要添加的菜品呢~")
    elif args and len(args) == 1:
        new_food = args[0]
    else:
        await add_group.finish("添加菜品参数错误~")
    
    user_id = str(event.user_id)
    logger.info(f"User {user_id} 添加了 {new_food} 至菜单")
    msg = eating_manager.add_group_food(new_food, event)

    await add_group.finish(msg)

@add_basic.handle()
async def _(bot: Bot, event: GroupMessageEvent, state: T_State):
    args = event.get_plaintext().strip().split()
    if not args:
        await add_basic.finish("还没输入你要添加的菜品呢~")
    elif args and len(args) == 1:
        new_food = args[0]
    else:
        await add_basic.finish("添加菜品参数错误~")
    
    user_id = str(event.user_id)
    logger.info(f"Superuser {user_id} 添加了 {new_food} 至基础菜单")
    msg = eating_manager.add_basic_food(new_food)

    await add_basic.finish(msg)

@remove_food.handle()
async def _(bot: Bot, event: GroupMessageEvent, state: T_State):
    args = event.get_plaintext().strip().split()
    if not args:
        await remove_food.finish("还没输入你要移除的菜品呢~")
    elif args and len(args) == 1:
        food_to_remove = args[0]
    else:
        await remove_food.finish("移除菜品参数错误~")
    
    user_id = str(event.user_id)
    logger.info(f"User {user_id} 从菜单移除了 {food_to_remove}")
    msg = eating_manager.remove_food(food_to_remove, event)

    await remove_food.finish(msg)

@show_group.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    msg = eating_manager.show_group_menu(event)
    await show_group.finish(msg)

@show_basic.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    msg = eating_manager.show_basic_menu()
    await show_basic.finish(msg)

@switch_greating.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    args = event.get_plaintext()
    if args[:2] == "开启":
        greating_helper.resume()
        msg = f"已开启按时吃饭小助手~"
    elif args[:2] == "关闭":
        greating_helper.pause()
        msg = f"已关闭按时吃饭小助手~"

    await switch_greating.finish(msg)

# 重置吃什么次数，包括夜宵
@eating_helper.scheduled_job("cron", hour="6,11,17,22", minute=0)
async def _():
    eating_manager.reset_eating()
    logger.info("今天吃什么次数已刷新")

# 早餐提醒
@greating_helper.scheduled_job("cron", hour=7, minute=0)
async def time_for_breakfast():
    bot = get_bot()
    msg = eating_manager.get2greating(Meals.BREAKFAST)
    if not GROUPS_ID:
        pass
    elif msg:
        for group_id in GROUPS_ID:
            await bot.send_group_msg(group_id=int(group_id), message=msg)
        
        logger.info(f"已群发早餐提醒")

# 午餐提醒
@greating_helper.scheduled_job("cron", hour=12, minute=0)
async def time_for_lunch():
    bot = get_bot()
    msg = eating_manager.get2greating(Meals.LUNCH)
    if not GROUPS_ID:
        pass
    elif msg:
        for group_id in GROUPS_ID:
            await bot.send_group_msg(group_id=int(group_id), message=msg)
        
        logger.info(f"已群发午餐提醒")

# 下午茶/摸鱼提醒
@greating_helper.scheduled_job("cron", hour=15, minute=0)
async def time_for_snack():
    bot = get_bot()
    msg = eating_manager.get2greating(Meals.SNACK)
    if not GROUPS_ID:
        pass
    elif msg:
        for group_id in GROUPS_ID:
            await bot.send_group_msg(group_id=int(group_id), message=msg)
        
        logger.info(f"已群发摸鱼提醒")

# 晚餐提醒
@greating_helper.scheduled_job("cron", hour=18, minute=0)
async def time_for_dinner():
    bot = get_bot()
    msg = eating_manager.get2greating(Meals.DINNER)
    if not GROUPS_ID:
        pass
    elif msg:
        for group_id in GROUPS_ID:
            await bot.send_group_msg(group_id=int(group_id), message=msg)
        
        logger.info(f"已群发晚餐提醒")

# 夜宵提醒
@greating_helper.scheduled_job("cron", hour=22, minute=0)
async def time_for_midnight():
    bot = get_bot()
    msg = eating_manager.get2greating(Meals.MIDNIGHT)
    if not GROUPS_ID:
        pass
    elif msg:
        for group_id in GROUPS_ID:
            await bot.send_group_msg(group_id=int(group_id), message=msg)
        
        logger.info(f"已群发夜宵提醒")


from nonebot.adapters.cqhttp import GroupMessageEvent, MessageSegment
import nonebot
import random
import os
from pathlib import Path
from typing import Optional
from enum import Enum

try:
    import ujson as json
except ModuleNotFoundError:
    import json

SUPERUSERS = nonebot.get_driver().config.superusers
_WHAT2EAT_PATH = nonebot.get_driver().config.what2eat_path
DEFAULT_PATH = os.path.join(os.path.dirname(__file__), "resource")
WHAT2EAT_PATH = DEFAULT_PATH if not _WHAT2EAT_PATH else _WHAT2EAT_PATH
_EATING_LIMIT = nonebot.get_driver().config.eating_limit
EATING_LIMIT = 6 if not _EATING_LIMIT else _EATING_LIMIT

'''
    需要群发问候的群组列表
'''
_GROUPS_ID = nonebot.get_driver().config.groups_id
GROUPS_ID = [] if not _GROUPS_ID else _GROUPS_ID

class Meals(Enum):
    BREAKFAST   = "breakfast"
    LUNCH       = "lunch"
    SNACK       = "snack"
    DINNER      = "dinner"
    MIDNIGHT    = "midnight"

class EatingManager:

    def __init__(self, path: Optional[Path]):
        self.greating_enbale = True     # 群聊按时吃饭提醒开关
        self._data = {}
        self._greating = {}
        if not path:
            data_file = Path(WHAT2EAT_PATH) / "data.json"
            greating_file = Path(WHAT2EAT_PATH) / "greating.json"
        else:
            data_file = path / "data.json"
            greating_file = path / "greating.json"
        
        self.data_file = data_file
        self.greating_file = greating_file
        if not data_file.exists():
            with open(data_file, "w", encoding="utf-8") as f:
                f.write(json.dumps(dict()))
                f.close()

        if data_file.exists():
            with open(data_file, "r", encoding="utf-8") as f:
                self._data = json.load(f)
        
        if not greating_file.exists():
            with open(greating_file, "w", encoding="utf-8") as f:
                f.write(json.dumps(dict()))
                f.close()

        if greating_file.exists():
            with open(greating_file, "r", encoding="utf-8") as f:
                self._greating = json.load(f)

        self._init_json()

    def _init_json(self) -> None:
        if "basic_food" not in self._data.keys():
            self._data["basic_food"] = []
        if "group_food" not in self._data.keys():
            self._data["group_food"] = {}
        if "eating" not in self._data.keys():
            self._data["eating"] = {}
        
        for meal in Meals:
            if meal.value not in self._greating.keys():
                self._greating[meal.value] = []
    
    def _init_data(self, group_id: str, user_id: str) -> None:
        '''
            初始化用户信息
        '''
        if group_id not in self._data["group_food"].keys():
            self._data["group_food"][group_id] = []
        if group_id not in self._data["eating"].keys():
            self._data["eating"][group_id] = {}
        if user_id not in self._data["eating"][group_id].keys():
            self._data["eating"][group_id][user_id] = 0

    def get2eat(self, event: GroupMessageEvent) -> str:
        '''
            今天吃什么
        '''
        user_id = str(event.user_id)
        group_id = str(event.group_id)

        self._init_data(group_id, user_id)
        if not self.eating_check(event):
            return random.choice(
                [
                    "你今天已经吃得够多了！",
                    "吃这么多的吗？",
                    "害搁这吃呢？不工作的吗？",
                    "再吃肚子就要爆炸咯~"
                ]
            )
        else:
            # 菜单全为空，建议避免["basic_food"]为空
            if len(self._data["basic_food"]) == 0 and len(self._data["group_food"][group_id]) == 0:
                return "还没有菜单呢，就先饿着肚子吧，请[添加 菜名]🤤"
            
            food_list = self._data["basic_food"].copy()
            if len(self._data["group_food"][group_id]) > 0:
                food_list.extend(self._data["group_food"][group_id])

            msg = "建议" + random.choice(food_list)
            self._data["eating"][group_id][user_id] += 1
            self.save()

            return msg
    
    '''
        检查菜品是否存在
        1:  存在于基础菜单
        2:  存在于群菜单
        0:  不存在
    '''
    def food_exists(self, _food_: str) -> int:
        for food in self._data["basic_food"]:
            if food == _food_:
                return 1

        for group_id in self._data["group_food"]:
            for food in self._data["group_food"][group_id]:
                if food == _food_:
                    return 2
        
        return 0

    '''
        检查是否吃饱
    '''
    def eating_check(self, event: GroupMessageEvent) -> bool:
        user_id = str(event.user_id)
        group_id = str(event.group_id)
        return False if self._data["eating"][group_id][user_id] >= EATING_LIMIT else True

    '''
        添加至群菜单中 GROUP_ADMIN | GROUP_OWNER 权限
    '''
    def add_group_food(self, new_food: str, event: GroupMessageEvent) -> str:
        user_id = str(event.user_id)
        group_id = str(event.group_id)

        self._init_data(group_id, user_id)
        status = self.food_exists(new_food)
        if status == 1:
            return f"{new_food} 已在基础菜单中~"
        elif status == 2:
            return f"{new_food} 已在群特色菜单中~"

        self._data["group_food"][group_id].append(new_food)
        self.save()
        return f"{new_food} 已加入群特色菜单~"

    '''
        添加至基础菜单 SUPERUSER 权限
    '''
    def add_basic_food(self, new_food: str) -> str:
        status = self.food_exists(new_food)
        if status == 1:
            return f"{new_food} 已在基础菜单中~"
        elif status == 2:
            return f"{new_food} 已在群特色菜单中~"

        self._data["basic_food"].append(new_food)
        self.save()
        return f"{new_food} 已加入基础菜单~"

    '''
        从基础菜单移除 SUPERUSER 权限
        从群菜单中移除 GROUP_ADMIN | GROUP_OWNER 权限
    '''
    def remove_food(self, food_to_remove: str, event: GroupMessageEvent) -> str:
        user_id = str(event.user_id)
        group_id = str(event.group_id)
        
        self._init_data(group_id, user_id)
        status = self.food_exists(food_to_remove)
        if not status:
            return f"{food_to_remove} 不在菜单中哦~"

        # 在群菜单
        if status == 2:
            self._data["group_food"][group_id].remove(food_to_remove)
            self.save()
            return f"{food_to_remove} 已从群菜单中删除~"
        # 在基础菜单
        else:
            if user_id not in SUPERUSERS:
                return f"{food_to_remove} 在基础菜单中，非超管不可操作哦~"
            else:
                self._data["basic_food"].remove(food_to_remove)
                self.save()
                return f"{food_to_remove} 已从基础菜单中删除~"    

    def reset_eating(self) -> None:
        '''
            重置三餐 eating times
        '''
        for group_id in self._data["eating"].keys():
            for user_id in self._data["eating"][group_id].keys():
                self._data["eating"][group_id][user_id] = 0
        
        self.save()

    def save(self) -> None:
        '''
            保存数据
        '''
        with open(self.data_file, 'w', encoding='utf-8') as f:
            json.dump(self._data, f, ensure_ascii=False, indent=4)
        
        with open(self.greating_file, 'w', encoding='utf-8') as f:
            json.dump(self._greating, f, ensure_ascii=False, indent=4)

    def show_group_menu(self, event: GroupMessageEvent) -> str:
        user_id = str(event.user_id)
        group_id = str(event.group_id)
        msg = []
        
        self._init_data(group_id, user_id)
        if len(self._data["group_food"][group_id]) > 0:
            msg += MessageSegment.text("---群特色菜单---\n")
            for food in self._data["group_food"][group_id]:
                msg += MessageSegment.text(f"{food}\n")
        
        return msg if len(msg) > 0 else "还没有群特色菜单呢，请[添加 菜名]~"

    def show_basic_menu(self) -> str:
        msg = []

        if len(self._data["basic_food"]) > 0:
            msg += MessageSegment.text("---基础菜单---\n")
            for food in self._data["basic_food"]:
                msg += MessageSegment.text(f"{food}\n")
        
        return msg if len(msg) > 0 else "还没有基础菜单呢，请[添加 菜名]~"

    '''
        干饭/摸鱼小助手：获取问候语，问候语为空返回None
    '''
    def get2greating(self, meal: Meals) -> Optional[str]:
        if len(self._greating.get(meal.value)) > 0:
            greatings = self._greating[meal.value]
            return random.choice(greatings)
        else:
            return None

    '''
        Reserved for next version
    '''
    def add_greating(self, new_greating: str, meal: Meals) -> str:
        self._greating[meal.value].append(new_greating)
        self.save()

        return f"{new_greating} 已加入 {meal.value} 问候~"

    '''
        Reserved for next version
    '''
    def remove_greating(self, remove_index: int, meal: Meals) -> str:
        greating = self._greating[meal.value].pop(remove_index)
        self.save()

        return f"{greating} 已从 {meal.value} 问候中移除~"


eating_manager = EatingManager(Path(WHAT2EAT_PATH))


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_what2eat']

package_data = \
{'': ['*'], 'nonebot_plugin_what2eat': ['resource/*']}

install_requires = \
['nonebot-plugin-apscheduler>=0.1.2,<0.2.0', 'ujson>=5.1.0,<6.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-what2eat',
    'version': '0.1.3',
    'description': 'What to eat today for your breakfast, lunch, dinner and even midnight snack!',
    'long_description': '<div align="center">\n\n# What to Eat\n\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable-next-line MD036 -->\n_🍔🌮🍜🍮🍣🍻🍩 今天吃什么 🍩🍻🍣🍮🍜🌮🍔_\n<!-- prettier-ignore-end -->\n\n</div>\n\n<p align="center">\n  \n  <a href="https://github.com/KafCoppelia/nonebot_plugin_what2eat/blob/main/LICENSE">\n    <img src="https://img.shields.io/badge/license-MIT-informational">\n  </a>\n  \n  <a href="https://github.com/nonebot/nonebot2">\n    <img src="https://img.shields.io/badge/nonebot2-2.0.0alpha.16-green">\n  </a>\n  \n  <a href="">\n    <img src="https://img.shields.io/badge/release-v0.1.3-orange">\n  </a>\n  \n</p>\n\n</p>\n\n## 版本\n\nv0.1.3 与**v0.2.3 for beta.1**版本同步\n\n⚠ 适配nonebot2-2.0.0beta.1；适配beta.1版本参见[beta.1分支](https://github.com/KafCoppelia/nonebot_plugin_what2eat/tree/beta.1)\n\n[更新日志](https://github.com/KafCoppelia/nonebot_plugin_what2eat/releases/tag/v0.1.3)\n\n## 安装\n\n1. 通过`pip`或`nb`，版本请指定`0.1.3`\n\n2. 数据默认位于`./resource/data.json`与`./resource/greating.json`，可通过设置`env`下`WHAT2EAT_PATH`更改；基础菜单、群特色菜单及群友询问Bot次数会记录在该文件中；\n\n## 功能\n\n1. 选择恐惧症？让Bot建议你今天吃什么！\n\n2. 每餐每个时间段询问Bot建议上限可通过`EATING_LIMIT`修改（默认6次），每日6点、11点、17点、22点（夜宵）自动刷新；\n\n3. 群管理可自行添加或移除群特色菜单（`data.json`下`[group_food][group_id]`）；超管可添加或移除基础菜单（`[basic_food]`）；\n\n4. 各群特色菜单相互独立；各群每个时间段询问Bot建议次数独立；Bot会综合各群菜单+基础菜单给出建议；\n\n5. 提醒按时吃饭小助手：每天7、12、15、18、22点群发**问候语**提醒群友按时吃饭/摸鱼，`GROUPS_ID`设置需要群发的群号列表，形如：\n\n    ```python\n    ["123456789", "987654321"]\n    ```\n\n6. **新增** 按时吃饭小助手全局开关；\n\n7. **新增** 查看群菜单与基础菜单命令分立；\n\n8. **新增** 吃什么帮助文案；\n\n9. **更新** 更新了预置基础菜单~~不想要可以移除~~；\n\n## 命令\n\n1. 吃什么：今天吃什么、中午吃啥、今晚吃啥、中午吃什么、晚上吃啥、晚上吃什么、夜宵吃啥……\n\n2. [管理或群主或超管] 添加或移除：添加/移除 菜名；\n\n3. 查看群菜单：菜单/群菜单/查看菜单；\n\n4. [超管] 添加至基础菜单：加菜 菜名；\n\n5. [超管] 查看基础菜单：基础菜单；\n\n6. [超管] 开启/关闭按时吃饭小助手：开启/关闭小助手；\n\n## 效果\n\n1. 案例1：\n\n    Q：今天吃什么\n\n    A：建议肯德基\n\n    （……吃什么*5）\n\n    Q：今晚吃什么\n\n    A：你今天已经吃得够多了！\n\n    Q：群菜单\n\n    A：\n\n    ---群特色菜单---\n\n    alpha\n\n    beta\n\n    gamma\n\n2. 案例2：\n\n    [群管] Q：添加 派蒙\n\n    A：派蒙 已加入群特色菜单~\n\n    [超管] Q：加菜 东方馅挂炒饭\n\n    A：东方馅挂炒饭 已加入基础菜单~\n\n    [群管] Q：移除 东方馅挂炒饭\n\n    A：东方馅挂炒饭 在基础菜单中，非超管不可操作哦~\n\n## 本插件改自：\n\n[HoshinoBot-whattoeat](https://github.com/pcrbot/whattoeat)',
    'author': 'KafCoppelia',
    'author_email': 'k740677208@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<4.0.0',
}


setup(**setup_kwargs)
